use serde_json::{to_string_pretty, Value};
use std::{backtrace::Backtrace, collections::HashMap, path::PathBuf, process::Stdio};
use tokio::io::AsyncWriteExt;

use crate::error::{Error, Result};

pub type StackMap = HashMap<String, String>;

pub fn parse_compose_stacks_output(bytes: Vec<u8>) -> Result<StackMap> {
    let json_output = serde_json::from_slice::<Value>(&bytes)?;
    let object = json_output.as_object().ok_or_else(|| Error::FormatError {
        message: "not an object".to_string(),
        backtrace: Backtrace::capture(),
    })?;

    let mut stacks = HashMap::new();
    for (key, value) in object.iter() {
        stacks.insert(key.to_string(), to_string_pretty(value)?);
    }

    Ok(stacks)
}

pub async fn export_compose_stacks(stacks: StackMap, output_dir: &PathBuf) -> Result<()> {
    // Create output directory if it doesn't exist
    if !output_dir.exists() {
        tokio::fs::create_dir_all(output_dir).await?;
    }

    // Create a docker-compose file for each stack
    for (stack_name, stack) in stacks.iter() {
        let stack_file_name = format!("{}.json", stack_name);
        let stack_file_path = output_dir.join(stack_file_name);
        tokio::fs::write(&stack_file_path, stack).await?;
    }

    Ok(())
}

pub enum ComposeOperation {
    Start,
    Stop,
    Restart,
    Down,
    Remove,
    Pause,
    Unpause,
    Kill,
    Create,
}

pub struct ComposeRunner {
    compose_binary_path: PathBuf,
}

impl ComposeRunner {
    pub fn new(compose_binary_path: PathBuf) -> Self {
        Self {
            compose_binary_path,
        }
    }

    pub async fn deploy_stack(&self, project_name: &str, stack_contents: &str) -> Result<()> {
        // Run the docker compose command and pipe stdout/stderr
        let mut docker_command = tokio::process::Command::new(&self.compose_binary_path)
            .args(&[
                "compose",
                "-p",
                project_name,
                "-f",
                "-",
                "up",
                "-d",
                "--no-color",
                "--remove-orphans",
            ])
            .stdin(Stdio::piped())
            .spawn()?;

        // Write contents to stdin
        docker_command
            .stdin
            .take()
            .ok_or(Error::ExecutionError {
                message: "no stdin".to_string(),
                backtrace: Backtrace::capture(),
            })?
            .write_all(stack_contents.as_bytes())
            .await?;

        let output = docker_command.wait_with_output().await?;
        if !output.status.success() {
            return Err(Error::DeployError {
                message: String::from_utf8_lossy(&output.stderr).to_string(),
                backtrace: Backtrace::capture(),
            });
        }

        Ok(())
    }

    pub async fn stack_op(&self, project_name: &str, operation: ComposeOperation) {
        let _ = tokio::process::Command::new(&self.compose_binary_path)
            .args(&[
                "compose",
                "-p",
                project_name,
                "-f",
                "-",
                match operation {
                    ComposeOperation::Start => "start",
                    ComposeOperation::Stop => "stop",
                    ComposeOperation::Restart => "restart",
                    ComposeOperation::Down => "restart",
                    ComposeOperation::Remove => "rm",
                    ComposeOperation::Pause => "pause",
                    ComposeOperation::Kill => "kill",
                    ComposeOperation::Create => "create",
                    ComposeOperation::Unpause => "unpause",
                },
            ])
            .stdin(Stdio::piped())
            .spawn()
            .unwrap()
            .wait()
            .await;
    }
}
