use crate::compose::{export_compose_stacks, parse_compose_stacks_output, StackMap};
use crate::error::{Error, Result};
use std::{backtrace::Backtrace, path::PathBuf};

macro_rules! run_deno {
    ($interpreter_path: expr) => {
        tokio::process::Command::new($interpreter_path)
            .env("NO_COLOR", "true")
            .env("DENO_NO_UPDATE_CHECK", "true")
    };
}

pub struct ScriptRunner {
    interpreter_path: PathBuf,
}

impl ScriptRunner {
    pub fn new(interpreter_path: PathBuf) -> Self {
        Self { interpreter_path }
    }

    pub async fn check_script(&self, script_path: &str) -> Result<()> {
        let output = run_deno!(&self.interpreter_path)
            .args(&["check", "-q", script_path])
            .output()
            .await?;

        if !output.status.success() {
            return Err(Error::ScriptError {
                message: String::from_utf8_lossy(&output.stderr).to_string(),
                backtrace: Backtrace::capture(),
            });
        }

        Ok(())
    }

    pub async fn run_script(&self, script_path: &str) -> Result<Vec<u8>> {
        let output = run_deno!(&self.interpreter_path)
            .args(&["run", "--no-prompt", "-q", "--no-check", script_path])
            .output()
            .await?;

        if !output.status.success() {
            return Err(Error::ExecutionError {
                message: String::from_utf8_lossy(&output.stderr).to_string(),
                backtrace: Backtrace::capture(),
            });
        }

        Ok(output.stdout)
    }

    pub async fn run_and_export(&self, script_path: &str, output_dir: &PathBuf) -> Result<()> {
        let output = self.run_script(script_path).await?;
        let stacks = parse_compose_stacks_output(output)?;
        export_compose_stacks(stacks, output_dir).await
    }

    pub async fn run_and_parse(&self, script_path: &str) -> Result<StackMap> {
        let output = self.run_script(script_path).await?;
        parse_compose_stacks_output(output)
    }
}
